#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from mailstoreapi.client import Client

class ManagementAPI(object):
    def __init__(self, user="admin", password="admin",
                 host="localhost", port=8463, ):
        """Main class for MailStoreApiClient.
        Calls methods from server and returns result.
        GetToken returns token result or -1.
        WaitForToken waits until token has answer."""
        self._client = Client(user, password, host, port)
        self.autoReturnHandledToken = True
        
    def checkAutoReturnHandledToken(self,  token,  autoReturnHandledToken):
        if autoReturnHandledToken is None:
            autoReturnHandledToken = self.autoReturnHandledToken
        if autoReturnHandledToken:
            token = self.WaitForToken(token)
        return token

    def GetToken(self, token):
        """Checks if the command finished executing
        and returns the result or -1 if it has not finished"""
        return self._client.getToken(token)

    def WaitForToken(self, token, wait=0.5):
        """Checks every {0} seconds if the command
        finished executing and returns the result when
        finished""".format(wait)
        return self._client.waitForToken(token, wait)

    def GetMethodList(self):
        """Returns every method which can be executed by
        the server"""
        return self._client.call("GetMethodList", {})
        
    def RequestCancellation(self, statusToken):
        """Cancells a Command which returned a token."""
        #statusToken
        return self._client.call("RequestCancellation",
                                {"statusToken":statusToken})

    def GetServerInfo(self):
        """Returns machinename and MailStore version of server"""
        return self._client.call("GetServerInfo", {})

    def GetActiveSessions(self):
        """Returns active users on server"""
        return self._client.call("GetActiveSessions", {})

    def GetChildFolders(self, folder=None, maxLevels=None):
        """Returns childfolders of a folder"""
        #folder
        #maxLevels
        return self._client.call("GetChildFolders", {"folder":folder,
                                                     "maxLevels":maxLevels})

    def GetStoreList(self):
        """Returns a list of dictionarys which describe the stores.
        Example:
        [{u'indexPath': u'./', u'requestedState': u'Normal', u'name': u'never', u'databasePath': u'./', u'contentPath': u'./', u'databaseName': u'MailStore', u'error': u"", u'type': u'FileSystemStandard', u'id': 43}]"""
        return self._client.call("GetStoreList", {})
        
    def SetStoreRequestedState(self, id, requestedState):
        """Sets the Store specified by id to the requested state.
        Example states are Normal or Disabled"""
        #id
        #state
        return self._client.call("SetStoreRequestedState", {"id":str(id),
                                                        "requestedState":requestedState})

    def DetachStore(self, id):
        """Detaches the store.
        The Store must be in the disabled state"""
        #id
        return self._client.call("DetachStore", {"id":str(id)})

    def RetryOpenStores(self):
        """Retry to open stores, if it failed before."""
        return self._client.call("RetryOpenStores", {})

    def RefreshAllStoreStatistics(self):
        return self._client.call("RefreshAllStoreStatistics", {})

    def CreateStore(self, name, type=None, databaseName=None, databasePath=None,
                    contentPath=None, indexPath=None, serverName=None,
                    userName=None, password=None, requestedState=None):
        """Creates a new store by the given arguments"""
        #name
        return self._client.call("CreateStore", {"name": name, "type": type,
                                           "databaseName": databaseName,
                                           "databasePath": databasePath,
                                           "contentPath": contentPath,
                                           "indexPath": indexPath,
                                           "serverName": serverName,
                                           "userName": userName,
                                           "password": password,
                                           "requestedState":requestedState})

    def AttachStore(self, name, databasePath, type=None, databaseName=None,
                    contentPath=None, indexPath=None, serverName=None,
                    userName=None, password=None, requestedState=None):
        """Attaches a store which has been detached before.
        CreateStore automatically attaches the store."""
        return self._client.call("AttachStore", {"name": name, "type": type,
                                           "databaseName":databaseName,
                                           "databasePath":databasePath,
                                           "contentPath":contentPath,
                                           "indexPath":indexPath,
                                           "serverName":serverName,
                                           "userName":userName,
                                           "password":password,
                                           "requestedState":requestedState})

    def ClearUserPrivilegesOnFolders(self, userName):
        """Removes the userspecific Privileges on a Folder"""
        return self._client.call("ClearUserPrivilegesOnFolders", {"userName":userName})

    def RenameStore(self, id, name):
        """Renames the store specified by the id"""
        #id
        #name
        return self._client.call("RenameStore", {"id":str(id), "name":name})

    def SetStoreProperties(self, id, type=None, databaseName=None, databasePath=None, contentPath=None,
                           indexPath=None, serverName=None, userName=None, password=None):
        """Changes or sets the stores properies"""
        #id
        return self._client.call("SetStoreProperties", {"id":str(id),
                                                        "type":type,
                                                        "databaseName":databaseName,
                                                        "databasePath":databasePath,
                                                        "contentPath":contentPath,
                                                        "indexPath":indexPath,
                                                        "serverName":serverName,
                                                        "userName":userName,
                                                        "password":password})

    def GetStoreIndexList(self, id):
        """Returns List of all Stores"""
        #id
        return self._client.call("GetStoreIndexList", {"id":str(id)})

    def RebuildStoreIndex(self, id, folder):
        """Rebuilds Store Index of the given id and folder"""
        #id
        #folder
        return self._client.call("RebuildStoreIndex", {"id":id, "folder":folder})

    def CompactStore(self, id):
        #id
        return self._client.call("CompactStore", {"id":str(id)})

    def CompactMasterDatabase(self, autoReturnHandledToken=None):
        #!Token
        token = self._client.call("CompactMasterDatabase", {})
        return self.checkAutoReturnHandledToken(token, autoReturnHandledToken)

    def UpgradeStore(self, id, autoReturnHandledToken=None):
        """Upgrades Store from older MailStore versions"""
        #id
        token = self._client.call("UpgradeStore", {"id":str(id)})
        return self.checkAutoReturnHandledToken(token, autoReturnHandledToken)

    def VerifyStore(self, id,  autoReturnHandledToken=None):
        """Verifies if Store is valid"""
        #id
        token = self._client.call("VerifyStore", {"id":str(id)})
        return self.checkAutoReturnHandledToken(token,  autoReturnHandledToken)

    def CreateUser(self, userName, privileges, fullName=None,
                   distinguishedName=None, authentication="integrated",
                   password=None):
        """Creates a new User with given Arguments"""
        return self._client.call("CreateUser", {"userName":userName,
                                                "privileges":privileges,
                                                "fullName":fullName,
                                                "distinguishedName": distinguishedName,
                                                "authentication":authentication,
                                                "password":password})

    def GetUserList(self):
        """Returns List of all Users"""
        return self._client.call("GetUserList", {})

    def GetUserInfo(self, userName):
        """Returns detailed information about a user"""
        #userName
        return self._client.call("GetUserInfo", {"userName":userName})

    def SetUserAuthentication(self, userName, authentication):
        """Sets how users will authenticate"""
        return self._client.call("SetUserAuthentication", {"userName":userName,
                                                           "authentication":authentication})

    def SetUserDistinguishedName(self, userName, distinguishedName=None):
        return self._client.call("SetUserDistinguishedName", {"userName":userName,
                                                              "distinguishedName":distinguishedName})

    def SetUserFullName(self, userName, fullName=None):
        return self._client.call("SetUserFullName",{"userName": userName,
                                                    "fullName": fullName})

    def RenameUser(self, oldUserName, newUserName):
        #oldUserName
        #newUserName
        return self._client.call("RenameUser", {"oldUserName":oldUserName, "newUserName":newUserName})

    def DeleteUser(self, userName):
        #userName
        return self._client.call("DeleteUser", {"userName":userName})
        
    def SetUserPassword(self, userName, password):
        #userName
        #password
        return self._client.call("SetUserPassword", {"userName":userName, "password":password})

    def SetUserEmailAddresses(self, userName, emailAddresses):
        #userName
        #emailAddresses split by ,
        if type(emailAddresses) in (list, tuple):
            emailAddresses = ",".join(emailAddresses)
        return self._client.call("SetUserEmailAddresses", {"userName":userName, "emailAddresses":emailAddresses})

    def SetUserPop3UserNames(self, userName, pop3UserNames):
        #userName
        #pop3UserNames split by ,
        if type(pop3UserNames) in (list, tuple):
            pop3UserNames = ",".join(pop3UserNames)
        return self._client.call("SetUserPop3UserNames", {"userName":userName, "pop3UserNames": pop3UserNames})

    def SetUserPrivileges(self, userName, privileges):
        #userName
        #privileges 
        if type(privileges) in (list,  tuple):
            privileges = ",".join(privileges)
        return self._client.call("SetUserPrivileges", {"userName":userName, "privileges": privileges})

    def SetUserPrivilegesOnFolder(self, userName, folder, privileges):
        """Sets custom user privileges for one folder"""
        #userName
        #folder
        #privileges 
        if type(privileges) in (list,  tuple):
            privileges = ",".join(privileges)
        return self._client.call("SetUserPrivilegesOnFolder", {"userName":userName, "folder":folder, "privileges":privileges})

    def SyncUsersWithDirectoryServices(self):
        return self._client.call("SyncUsersWithDirectoryServices", {})

    def MoveFolder(self, fromFolder, toFolder):
        return self._client.call("MoveFolder", {"fromFolder":fromFolder,
                                                "toFolder":toFolder})
                                                
    def DeleteEmptyFolders(self,  folder=None):
        return self._client.call("DeleteEmptyFolders", {"folder":folder})
    
    def MaintainFileSystemDatabases(self, autoReturnHandledToken=None):
        token = self._client.call("MaintainFileSystemDatabases", {})
        return self.checkAutoReturnHandledToken(token,  autoReturnHandledToken)
